window.Gleam.initSearch([{"doc":"rememo","title":"rememo","content":"# rememo\n\n[![Package Version](https://img.shields.io/hexpm/v/rememo)](https://hex.pm/packages/rememo)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/rememo/)\n\nThis is a basic [memoization](https://en.wikipedia.org/wiki/Memoization) method for Gleam on both the Erlang and Javascript targets, for caching and reusing the results of function calls when possible.  \n\nAdding to and reading from the memoization cache incurs some overhead, but in situations that call for it, like dynamic programming problems where the same basic subproblem recurs hundreds or thousands of times, the tradeoff is worthwhile.  Always benchmark your code when in doubt.\n\n```sh\ngleam add rememo\n```\n\n```gleam\nimport rememo/memo \nimport gleam/io\n\npub fn main() {\n  // Start the actor that holds the cached values\n  // for the duration of this block\n  use cache &lt;- memo.create()\n  fib(300, cache)\n  |&gt; io.debug\n}\n\nfn fib(n, cache) {\n  // Check if a value exists for the key n\n  // Use it if it exists, update the cache if it doesn&#39;t\n  use &lt;- memo.memoize(cache, n)\n  case n {\n    1 | 2 -&gt; 1\n    n -&gt; fib(n - 1, cache) + fib(n - 2, cache)\n  }\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/rememo&gt;.\n\n## Contributing and development\n\nSuggestions and pull requests are welcome.\n","url":"index.html"},{"doc":"rememo/memo","title":"create","content":"pub fn create(apply fun: fn(Set(a, b)) -&gt; c) -&gt; c\n Make a new memoization cache, of the appropriate type for the build target.\n Pass this cache to the function you want to memoize.\n \n This is best used with a `use` expression:\n \n use cache &lt;- create()\n f(a, b, c, cache)\n \n \nSynonyms:\nrememo/memo.create\nrememo/memo create","url":"rememo/memo.html#create"},{"doc":"rememo/memo","title":"get","content":"pub fn get(from cache: Set(a, b), fetch key: a) -&gt; Result(b, Nil)\n Manually look up a value from the memoization cache for a given key.\n Useful if you want to also return intermediate results as well as a final result, for example.\nSynonyms:\nrememo/memo.get\nrememo/memo get","url":"rememo/memo.html#get"},{"doc":"rememo/memo","title":"memoize","content":"pub fn memoize(\n  with cache: Set(a, b),\n  this key: a,\n  apply fun: fn() -&gt; b,\n) -&gt; b\n Look up the value associated with the given key in the memoization cache,\n and return it if it exists.  If it doesn&#39;t exist, evaluate the callback function\n and update the cache with the value it returns.\n \n This works well with a `use` expression:\n \n fn f(a, b, c, cache) {\n   use &lt;- memoize(cache, #(a, b, c))\n   // function body goes here\n }\n \n \nSynonyms:\nrememo/memo.memoize\nrememo/memo memoize","url":"rememo/memo.html#memoize"},{"doc":"rememo/memo","title":"set","content":"pub fn set(\n  in cache: Set(a, b),\n  for key: a,\n  insert value: b,\n) -&gt; Nil\n Manually add a key-value pair to the memoization cache.\n Useful if you need to pre-seed the cache with a starting value, for example.\nSynonyms:\nrememo/memo.set\nrememo/memo set","url":"rememo/memo.html#set"},{"doc":"rememo/memo","title":"rememo/memo","content":" The memoization implementation depends on the build target.\n \n * For the Erlang build target, the cache is a [Erlang Term Storage \n database](https://www.erlang.org/doc/apps/erts/persistent_term.html).\n * For the Javascript build target, the cache is a [mutable Javascript map](\n https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Map).","url":"rememo/memo.html"}]);